/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcfileeditor.h"

#include "../../kdcconfig.h"
#include "../../kdcsubtitlestream.h"
#include "../../kdcdisc.h"
#include "../../lib/kdclib.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QString>

#include <KFileDialog>
#include <KUrl>
#include <KIcon>
#include <KConfigGroup>
#include <KLocale>

KDCFileEditor::KDCFileEditor( QWidget *parent )
	: QSplitter( Qt::Horizontal, parent )
{
	p_movieList = new KDCMovieList;
	p_subList = new KDCSubtitleList;
	p_add = new KPushButton ( KIcon("document-open"), i18n("Add") );
	p_add->setToolTip( i18n("Add subtitle file") );
	connect( p_add, SIGNAL( clicked() ), this, SLOT( open() ) );
	p_del = new KPushButton( KIcon("edit-delete-shred"), i18n("Delete") );
	p_del->setToolTip( i18n("Remove subtitle file") );
	connect( p_del, SIGNAL( clicked() ), this, SLOT( del() ) );

	QHBoxLayout *hLayout = new QHBoxLayout;
	hLayout->addWidget( p_add );
	hLayout->addWidget( p_del );
	hLayout->addStretch();

	QWidget *widget2 = new QWidget;
	QVBoxLayout *vLayout = new QVBoxLayout( widget2 );
	vLayout->addLayout( hLayout );
	vLayout->addWidget( p_subList );

	addWidget( p_movieList );
	setStretchFactor( 0, 1 );
	addWidget( widget2 );
	setStretchFactor( 1, 2 );

	connect( p_movieList, SIGNAL( currentMovieChanged( KDCMovie * ) ), this, SLOT( refresh( KDCMovie * ) ) );
	connect( p_subList, SIGNAL( currentSubtitleStreamChanged( KDCSubtitleStream * ) ), this, SLOT( subtitleStreamChanged( KDCSubtitleStream * ) ) );
}

KDCFileEditor::~KDCFileEditor()
{

}

void KDCFileEditor::refresh( KDCMovie *movie )
{
	if ( movie )
	{
		p_add->setEnabled( TRUE );
		p_subList->setCurrentMovie( movie );
	}
	else
	{
		p_del->setEnabled( FALSE );
		p_add->setEnabled( FALSE );
	}
}

void KDCFileEditor::setCurrentMovie( KDCMovie *movie )
{
	p_subList->clean();
	p_movieList->setCurrentMovie( movie );
	p_subList->setCurrentMovie( movie );
}

void KDCFileEditor::setCurrentMovie( KDCMovieGroup *movie )
{
	p_subList->clean();
	p_movieList->clean();
	p_movieList->setCurrentMovie( movie );
}

void KDCFileEditor::subtitleStreamChanged( KDCSubtitleStream *sub )
{
	if ( sub )
		p_del->setEnabled( TRUE );
	else
		p_del->setEnabled( FALSE );
}

void KDCFileEditor::clean()
{
	qDebug( "KDCFileEditor::clean()" );
	p_subList->clean();
	p_movieList->clean();
	refresh( NULL );
}

void KDCFileEditor::open()
{
	qDebug( "KDCFileEditor::open()::begin::movie = %d", KDCLib::isNull( p_movieList->currentMovie() ) );
	KDCMovie *movie = p_movieList->currentMovie();
	if ( movie->subtitleStreamGroup()->count() == movie->disc()->maxSubtitleSreams() )
	{
		emit infoMessage(	QString( i18n("Max. subtitle streams in ") +
											movie->disc()->name() + i18n(" is ") + "%1" )
											.arg( movie->disc()->maxSubtitleSreams() ) );
		return;
	}

  KConfig config("kdvdcreator");
	KConfigGroup general( &config, "General" );
	QString filter( "*.txt *.sub *.srt|All Subtitle Files" );
	QString dir = general.readEntry( "LastSubtitlesDir", QDir::homePath() );
	QStringList files = KFileDialog::getOpenFileNames( KUrl( dir ), filter, 0,
																										i18n("Select one or more subtitle files to open for current movie") );
	if ( !files.isEmpty() )
	{
		double fps = movie->videoStream()->metaData()->fps();
		general.writeEntry( "LastSubtitlesDir", QFileInfo( files.first() ).path() );
		general.config()->sync();

		for ( int i=0; i<files.size(); ++i )
		{
			if ( movie->subtitleStreamGroup()->count() == movie->disc()->maxSubtitleSreams() )
			{
				emit infoMessage( i18n("Some subtitles could not be loaded") );
				return;
			}
			movie->subtitleStreamGroup()->addStream( new KDCSubtitleStream( files.at( i ), fps ) );
		}
		emit infoMessage( i18n("All files added sucesfully"), KDCInfoStatusBar::Info );
		p_subList->refresh();
	}
	qDebug( "KDCFileEditor::open()::end" );
}

void KDCFileEditor::del()
{
	qDebug( "KDCFileEditor::del()::begin" );
	int idx = p_subList->indexOfTopLevelItem( p_subList->currentItem() );
	KDCMovie *movie = p_movieList->currentMovie();
	delete movie->subtitleStreamGroup()->removeStream( idx );
	qDebug( "KDCFileEditor::del()::end" );
}
